import styled from '@emotion/styled'
import { Button, message, Tag, Image } from 'antd'
import Countdown from 'antd/lib/statistic/Countdown'
import _ from 'lodash'
import moment from 'moment'
import React, { useCallback, useEffect, useMemo, useState } from 'react'
import 'react-alice-carousel/lib/alice-carousel.css'
import { useDispatch, useSelector } from 'react-redux'
import { useAppDispatch } from '../../../store'
import { Auction, AuctionState } from '../../../store/model/Auction'
import { auctionActions } from '../../../store/reducer/aunctionReducer'
import { bidActions } from '../../../store/reducer/bidReducer'
import { web3Selector } from '../../../store/selector/web3Selector'
import { ProductDescriptionView } from '../../product/ProductDescriptionView'
import KakaoMapScript from '../../shared/KakaoMapScript'
import { BidModal } from './BidModal'

interface Props {
    auction: Auction
}

const handleDragStart = (e: { preventDefault: () => any }) => e.preventDefault()

function AuctionDetailView({ auction }: Props) {
    const { product, bids, endAt, auctionState } = auction
    const dispatch = useAppDispatch()
    const email = useSelector(web3Selector.email)
    const [isOpenBid, setIsOpenBid] = useState(false)
    const userId = useSelector(web3Selector.userId)

    const { address, imageUrls } = product
    const handleBid = useCallback((price: number) => {
        userId &&
            email &&
            dispatch(
                bidActions.createBid({
                    userId,
                    price,
                    auctionId: auction.auctionId,
                    account: email,
                }),
            ).then(() => {
                message.success('성공적으로 경매에 참가하였습니다.')
                dispatch(auctionActions.getBiddedAuctions({ userId }))
                dispatch(auctionActions.getCreatedAuctions({ userId }))
                dispatch(auctionActions.getAuctions())
                setIsOpenBid(false)
            })
    }, [])
    const currentPrice = _.get(bids, 0)?.price ?? 0

    useEffect(() => {
        KakaoMapScript(address)
    }, [address])

    const items = useMemo(
        () =>
            imageUrls.map((image) => (
                <Image
                    src={image ?? '/anjun-logo.png'}
                    alt="logo"
                    width={200}
                    height={200}
                />
            )),
        [imageUrls],
    )

    const isAuctionEnd = useMemo(() => moment().isAfter(moment(endAt)), [endAt])

    const handleOpenBid = useCallback(() => {
        setIsOpenBid(true)
    }, [])

    const handleCompleteAuction = useCallback(() => {
        dispatch(
            auctionActions.completeAuction({
                auctionId: auction.auctionId,
                account: email ?? '',
            }),
        )
    }, [email])

    return (
        <div
            style={{
                display: 'flex',
                flexDirection: 'column',
                width: '100%',
                overflow: 'scroll',
            }}
        >
            <Header>
                {isAuctionEnd ? (
                    <Tag color="red">경매 종료됨</Tag>
                ) : (
                    <Countdown
                        title="경매 종료 남은시간"
                        value={moment(endAt).valueOf()}
                        format="HH:mm:ss:SSS"
                    />
                )}
                {isAuctionEnd && auctionState === AuctionState.PROCESSING && (
                    <Button type="primary" onClick={handleCompleteAuction} />
                )}
                {!isAuctionEnd && auctionState === AuctionState.PROCESSING && (
                    <Button type="primary" onClick={handleOpenBid}>
                        입찰하기
                    </Button>
                )}
            </Header>
            <div
                id="myMap"
                style={{
                    width: '100%',
                    height: '300px',
                }}
            />

            <BidModal
                auction={auction}
                isModalVisible={isOpenBid}
                handleOk={handleBid}
                handleCancel={() => setIsOpenBid(false)}
            />

            <ImageContainer>{items}</ImageContainer>

            <ProductDescriptionView product={product} />
        </div>
    )
}

export default AuctionDetailView

const Header = styled.div({
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'space-between',
    padding: 12,
})

const ImageContainer = styled.div({
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'flex-start',
    marginTop: 6,
    marginBottom: 6,
    [`& > * + *`]: {
        marginLeft: 6,
    },
})
