import styled from '@emotion/styled'
import { Button, DatePicker, DatePickerProps, Form, Input, Select } from 'antd'
import { useForm, useWatch } from 'antd/lib/form/Form'
import _ from 'lodash'
import moment from 'moment'
import { useCallback, useEffect, useMemo, useState } from 'react'
import { useSelector } from 'react-redux'
import { useNavigate } from 'react-router-dom'
import { useAppDispatch } from '../../store'
import { Product } from '../../store/model/Product'
import { auctionActions } from '../../store/reducer/aunctionReducer'
import { productActions } from '../../store/reducer/productReducer'
import { productSelector } from '../../store/selector/productSelector'
import { web3Selector } from '../../store/selector/web3Selector'
import { ProductDetailView } from '../product/ProductDetailView'
import KakaoMapScript from '../shared/KakaoMapScript'
import { auction } from '../../service/web3'

const { Option } = Select

export interface CreateAuction {
    userId: number
    productId: number
    endAt: moment.Moment
    startAt: moment.Moment
    title: string
    description: string
}

export function CreateAuctionView() {
    const navigate = useNavigate()
    const dispatch = useAppDispatch()
    const user = useSelector(web3Selector.user)
    const email = useSelector(web3Selector.email)

    const products = useSelector(productSelector.products)
    const [product, setProduct] = useState<Product>()
    const [form] = useForm<CreateAuction>()

    const productId = useWatch('productId', form)

    useEffect(() => {
        setProduct(
            _.find(products, (_product) => _product.productId === productId),
        )
    }, [products, productId])

    useEffect(() => {
        if (product) {
            KakaoMapScript(product.address)
        }
    }, [product])

    useEffect(() => {
        user !== undefined &&
            dispatch(productActions.getProducts({ userId: user?.userId }))
    }, [user])

    useEffect(() => {
        if (products !== undefined && products.length > 0)
            form.setFieldsValue({ productId: products[0].productId })
    }, [products])

    const handleCreateAuction = useCallback((formData: CreateAuction) => {
        if (user?.userId !== undefined) {
            dispatch(
                auctionActions.createAuction({
                    ...formData,
                    startAt: moment().toISOString(),
                    endAt: formData.endAt.toISOString(),
                    userId: user.userId,
                    account: email ?? '',
                }),
            ).then(() => {
                navigate('/auction/my')
            })
        }
    }, [])

    const handleChangeEndAt: DatePickerProps['onChange'] = (
        date,
        dateString,
    ) => {
        date && form.setFieldsValue({ endAt: date })
    }

    const options = useMemo(
        () =>
            products.map(({ productId, description }) => (
                <Option key={productId} value={productId}>
                    {description}
                </Option>
            )),
        [products],
    )

    const handleTest = useCallback(() => {
        // @ts-ignore
        auction.methods.add(1, 10, email ?? '').send({
            from: email ?? '',
            gas: 1500000,
            gasPrice: '30000000000000',
        })
    }, [email])

    return (
        <FormContainer>
            {product && <ProductDetailView product={product} />}
            <FormWrapper>
                <Form
                    labelCol={{ span: 12 }}
                    wrapperCol={{ span: 16 }}
                    form={form}
                    onFinish={handleCreateAuction}
                >
                    <Form.Item name="title" label="제목">
                        <Input />
                    </Form.Item>
                    <Form.Item name="description" label="설명">
                        <Input />
                    </Form.Item>
                    <Form.Item name="productId" label="매물">
                        <Select>{options}</Select>
                    </Form.Item>

                    <Form.Item name="endAt" label="경매 종료시간">
                        <DatePicker onChange={handleChangeEndAt} />
                    </Form.Item>

                    <Form.Item name="startAt" noStyle />

                    <Form.Item wrapperCol={{ offset: 12, span: 16 }}>
                        <Button type="primary" htmlType="submit">
                            경매 생성하기
                        </Button>
                        {/* <Button onClick={handleTest}>경매 생성하기</Button> */}
                    </Form.Item>
                </Form>
            </FormWrapper>
        </FormContainer>
    )
}

const FormContainer = styled.div({
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    overflow: 'scroll',
})

const FormWrapper = styled.div({
    width: 700,
    paddingRight: 120,
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
})
