import styled from '@emotion/styled'
import {
    Button,
    DatePicker,
    Form,
    Input,
    InputNumber,
    Modal,
    Typography,
} from 'antd'
import { useForm } from 'antd/lib/form/Form'
import { Moment } from 'moment'
import { useCallback, useEffect, useState } from 'react'
import { useSelector } from 'react-redux'
import DaumPostcode from 'react-daum-postcode'
import { useNavigate } from 'react-router-dom'
import _ from 'lodash'
import { useAppDispatch } from '../../store'
import { initialProduct, Product } from '../../store/model/Product'
import { productActions } from '../../store/reducer/productReducer'
import { web3Selector } from '../../store/selector/web3Selector'
import { ImageAccept } from '../../store/model/FileType'
import { UploadComponent } from './Upload'
import KakaoMapScript from '../shared/KakaoMapScript'

export function CreateProductView() {
    const navigate = useNavigate()
    const dispatch = useAppDispatch()
    const user = useSelector(web3Selector.user)
    const [form] = useForm<Product>()
    const imageUrls = Form.useWatch('imageUrls', form)
    const address = Form.useWatch('address', form)

    const [isOpenPost, setIsOpenPost] = useState(false)

    useEffect(() => {
        KakaoMapScript(address)
    }, [address])
    const handleOpenPost = () => {
        setIsOpenPost(true)
    }

    const handleClosePost = () => {
        setIsOpenPost(false)
    }

    const onCompletePost = (data: {
        address: string
        bname: string
        buildingName: string
        addressType: string
    }) => {
        let fullAddr = data.address
        let extraAddr = ''

        if (data.addressType === 'R') {
            if (data.bname !== '') {
                extraAddr += data.bname
            }
            if (data.buildingName !== '') {
                extraAddr +=
                    extraAddr !== ''
                        ? `, ${data.buildingName}`
                        : data.buildingName
            }
            fullAddr += extraAddr !== '' ? ` (${extraAddr})` : ''
        }
        form.setFieldsValue({ address: fullAddr })
        handleClosePost()
    }

    const handleCreateProduct = useCallback((product: Product) => {
        console.log('product', product)
        if (user?.userId !== undefined) {
            dispatch(
                productActions.createProduct({ product, userId: user?.userId }),
            ).then(() => {
                navigate('/product')
            })
        }
    }, [])

    const handleUploadFile = useCallback(
        (file: File) =>
            dispatch(productActions.uploadImage({ file })).then(
                (action) => action.payload as string,
            ),
        [],
    )

    const handleImageUrlsChange = useCallback(
        // eslint-disable-next-line no-shadow
        (imageUrls: Array<string>) => {
            console.log(imageUrls, 'imageUrls')
            form.setFieldsValue({
                imageUrls: imageUrls.map((imageUrl) =>
                    _.get(imageUrl, 'imageUrl'),
                ),
            })
        },
        [],
    )

    return (
        <FormContainer>
            <div
                id="myMap"
                style={{
                    width: '400px',
                    height: '400px',
                }}
            />
            <Form
                labelCol={{ span: 8 }}
                wrapperCol={{ span: 16 }}
                style={{ height: '100%' }}
                form={form}
                initialValues={initialProduct}
                onFinish={handleCreateProduct}
            >
                <Form.Item name="name" label="이름">
                    <Input />
                </Form.Item>

                <Form.Item name="description" label="설명">
                    <Input />
                </Form.Item>

                <Form.Item name="address" noStyle />
                <Form.Item label="주소">
                    <Typography.Text style={{ display: 'block' }}>
                        {address}
                    </Typography.Text>
                    <Button onClick={handleOpenPost}>우편번호 검색</Button>

                    <Modal
                        title="우편번호 검색"
                        visible={isOpenPost}
                        onCancel={handleClosePost}
                        footer={null}
                        destroyOnClose
                    >
                        <DaumPostcode
                            style={{
                                display: 'block',
                                position: 'relative',
                                width: '400px',
                                height: '600px',
                                padding: '7px',
                            }}
                            onComplete={onCompletePost}
                        />
                    </Modal>
                </Form.Item>
                <Form.Item name="detailAddress" label="상세주소">
                    <Input />
                </Form.Item>
                <Form.Item noStyle name="imageUrls" />
                <Form.Item label="이미지">
                    <UploadComponent
                        accept={ImageAccept}
                        listType="picture-card"
                        maxCount={3}
                        multiple
                        imageUrls={imageUrls ?? []}
                        onImageUrlsChange={handleImageUrlsChange}
                        onFileUpload={handleUploadFile}
                    />
                </Form.Item>
                <Form.Item name="buildingFormat" label="건물형태">
                    <Input />
                </Form.Item>
                <Form.Item name="roomArchitecture" label="방구조">
                    <Input />
                </Form.Item>
                <Form.Item name="area" label="전용면적">
                    <InputNumber
                        addonAfter={
                            <span>
                                m<sup>2</sup>
                            </span>
                        }
                    />
                </Form.Item>
                <Form.Item name="availableDate" noStyle />

                <Form.Item wrapperCol={{ offset: 8, span: 16 }}>
                    <Button type="primary" htmlType="submit">
                        매물생성하기
                    </Button>
                </Form.Item>
            </Form>
        </FormContainer>
    )
}

const FormContainer = styled.div({
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    overflow: 'scroll',
})
