import { useCallback, useMemo, useState } from 'react'
import { Modal, Upload, UploadProps, message } from 'antd'
import {
    RcFile,
    UploadChangeParam,
    UploadFile,
} from 'antd/lib/upload/interface'
import styled from '@emotion/styled'
import { UploadRequestOption } from 'rc-upload/lib/interface'
import { PlusOutlined } from '@ant-design/icons'
import { v4 as uuid } from 'uuid'
import _ from 'lodash'
import {
    imageUrlsToUploadFile,
    uploadFilesToImageUrl,
} from '../../utils/imageUtils'
import { FileType } from '../../store/model/FileType'

type ImageUrls = Array<string>

export interface Props extends Omit<UploadProps, 'accept'> {
    imageUrls: ImageUrls
    // eslint-disable-next-line no-unused-vars
    onFileUpload: (image: File) => Promise<string>
    // eslint-disable-next-line no-unused-vars
    onImageUrlsChange: (imageUrls: ImageUrls) => void
    // eslint-disable-next-line react/require-default-props
    accept?: FileType[]
}

export function UploadComponent({
    imageUrls,
    onFileUpload,
    onImageUrlsChange,
    maxCount = 1,
    accept,
    ...otherProps
}: Props) {
    const [previewVisible, setPreviewVisible] = useState(false)
    const [previewImage, setPreviewImage] = useState('')
    const [previewTitle, setPreviewTitle] = useState('')
    const uploadFiles = useMemo(
        () => imageUrlsToUploadFile(imageUrls),
        [imageUrls],
    )

    const handlePreview = useCallback(async (file: UploadFile) => {
        setPreviewVisible(true)
        setPreviewImage(file.response.imageUrl)
        setPreviewTitle(
            file.name ||
                file.response.imageUrl.substring(
                    file.response.imageUrl.lastIndexOf('/') + 1,
                ),
        )
    }, [])

    const handleChange = useCallback(
        ({ fileList }: UploadChangeParam) => {
            onImageUrlsChange(uploadFilesToImageUrl(fileList))
        },
        [onImageUrlsChange],
    )

    const handleCustomRequest = useCallback(
        async ({ onSuccess, onError, file }: UploadRequestOption) => {
            const name = _.get(file, 'name', uuid())

            try {
                const customFile = new File([file], name)
                const res = await onFileUpload(customFile)

                onSuccess?.(res)
            } catch (error) {
                onError?.(new Error('파일 업로드에 실패했습니다.'))
            }
        },
        [onFileUpload],
    )

    const handleBeforeUpload = useCallback(
        (file: RcFile) => {
            const type = _.get(file, 'type')

            const isAcceptFileType =
                accept === undefined || _.includes(accept, type)

            if (!isAcceptFileType) {
                message.error(
                    `업로드 할 수 없는 파일 타입입니다. ${accept?.join(
                        ',',
                    )}\n 에 해당하는 파일 타입만 업로드 할 수 있습니다.`,
                )
            }

            return isAcceptFileType === false
                ? Upload.LIST_IGNORE
                : isAcceptFileType
        },
        [accept],
    )

    const handleCancel = useCallback(() => {
        setPreviewVisible(false)
    }, [])

    const uploadButton = useMemo(
        () => (
            <div>
                <PlusOutlined />
                <UploadButtonText>Upload</UploadButtonText>
            </div>
        ),
        [],
    )

    return (
        <>
            <Upload
                accept={accept?.join(',')}
                beforeUpload={handleBeforeUpload}
                onChange={handleChange}
                onPreview={handlePreview}
                customRequest={handleCustomRequest}
                maxCount={maxCount}
                {...otherProps}
            >
                {uploadFiles.length >= maxCount ? null : uploadButton}
            </Upload>
            <Modal
                visible={previewVisible}
                title={previewTitle}
                footer={null}
                onCancel={handleCancel}
            >
                <Image alt={previewTitle} src={previewImage} />
            </Modal>
        </>
    )
}

const Image = styled.img({
    width: '100%',
})

const UploadButtonText = styled.div({
    marginTop: 8,
})
