import {
    createAsyncThunk,
    createEntityAdapter,
    createSlice,
    EntityState,
    PayloadAction,
} from '@reduxjs/toolkit'
import _ from 'lodash'
import { AuctionAPI } from '../api/AuctionAPI'
import { Auction, AuctionState, CreateAuction } from '../model/Auction'

export const completeAuction = createAsyncThunk(
    'POST/completeAuction',
    async ({ account, auctionId }: { account: string; auctionId: number }) =>
        AuctionAPI.completeAuction({ account, auctionId }),
)

export const createAuction = createAsyncThunk(
    'POST/createAuction',
    async (data: CreateAuction) => AuctionAPI.createAuction(data),
)

export const getAuctions = createAsyncThunk('GET/getAuctions', async () =>
    AuctionAPI.getAuctions({}),
)

export const getBiddedAuctions = createAsyncThunk(
    'GET/getBiddedAuctions',
    async (data: { userId: number }) => AuctionAPI.getBiddedAuctions(data),
)

export const getCreatedAuctions = createAsyncThunk(
    'GET/getCreatedAuctions',
    async (data: { userId: number }) => AuctionAPI.getCreatedAuctions(data),
)

export const auctionAdapter = createEntityAdapter<Auction>({
    selectId: (auction) => auction.auctionId,
})

interface InitialState {
    auctions: EntityState<Auction>
    biddedAuctions: EntityState<Auction>
    createdAuctions: EntityState<Auction>
    isLoading: boolean
    selectedAuctionId: number
}

const initialState: InitialState = {
    selectedAuctionId: 0,
    auctions: auctionAdapter.getInitialState(),
    biddedAuctions: auctionAdapter.getInitialState(),
    createdAuctions: auctionAdapter.getInitialState(),
    isLoading: false,
}

const auctionSlice = createSlice({
    name: 'auction',
    initialState,
    reducers: {
        selectAuction: (state, action: PayloadAction<number>) => {
            state.selectedAuctionId = action.payload
        },
    },
    extraReducers: (builder) => {
        builder
            .addCase(getAuctions.pending, (state) => {
                state.isLoading = true
            })
            .addCase(getAuctions.fulfilled, (state, action) => {
                state.isLoading = false
                auctionAdapter.setAll(state.auctions, action.payload)
            })
            .addCase(getCreatedAuctions.pending, (state) => {
                state.isLoading = true
            })
            .addCase(getCreatedAuctions.fulfilled, (state, action) => {
                state.isLoading = false
                auctionAdapter.setAll(state.createdAuctions, action.payload)
            })
            .addCase(getBiddedAuctions.pending, (state) => {
                state.isLoading = true
            })
            .addCase(getBiddedAuctions.fulfilled, (state, action) => {
                state.isLoading = false
                auctionAdapter.setAll(state.biddedAuctions, action.payload)
            })
            .addCase(createAuction.pending, (state) => {
                state.isLoading = true
            })
            .addCase(createAuction.fulfilled, (state) => {
                state.isLoading = false
            })
    },
})

export const auctionActions = {
    ...auctionSlice.actions,
    createAuction,
    getAuctions,
    getCreatedAuctions,
    getBiddedAuctions,
    completeAuction,
}

export default auctionSlice
